Re: [PATCH 2/4] mm: compaction: convert to folio_isolate_movable()

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

 



Hi Kefeng,

On 2024/8/26 12:01, Kefeng Wang wrote:
The tail page will always fail to isolate for non-lru-movable and
LRU page in isolate_migratepages_block(), so move PageTail check
ahead, then convert to use folio_isolate_movable() helper and more
folios.

Signed-off-by: Kefeng Wang <wangkefeng.wang@xxxxxxxxxx>
---
  mm/compaction.c | 32 +++++++++++++++++---------------
  1 file changed, 17 insertions(+), 15 deletions(-)

diff --git a/mm/compaction.c b/mm/compaction.c
index a2b16b08cbbf..aa2e8bb9fa58 100644
--- a/mm/compaction.c
+++ b/mm/compaction.c
@@ -1074,39 +1074,41 @@ isolate_migratepages_block(struct compact_control *cc, unsigned long low_pfn,
  			}
  		}
+ if (PageTail(page))
+			goto isolate_fail;
+
+		folio = page_folio(page);

I wonder if this is stable. Before page_folio(), it does not hold a reference on the page, so seems we should re-check the folio still contains this page after gaining a reference on the folio?

  		/*
  		 * Check may be lockless but that's ok as we recheck later.
-		 * It's possible to migrate LRU and non-lru movable pages.
-		 * Skip any other type of page
+		 * It's possible to migrate LRU and non-lru movable folioss.
+		 * Skip any other type of folios.
  		 */
-		if (!PageLRU(page)) {
+		if (!folio_test_lru(folio)) {
  			/*
 			 * __PageMovable can return false positive so we need
 			 * to verify it under page_lock.
  			 */
-			if (unlikely(__PageMovable(page)) &&
-					!PageIsolated(page)) {
+			if (unlikely(__folio_test_movable(folio)) &&
+					!folio_test_isolated(folio)) {
  				if (locked) {
  					unlock_page_lruvec_irqrestore(locked, flags);
  					locked = NULL;
  				}
- if (isolate_movable_page(page, mode)) {
-					folio = page_folio(page);
+				if (folio_isolate_movable(folio, mode))
  					goto isolate_success;
-				}
  			}
goto isolate_fail;
  		}
/*
-		 * Be careful not to clear PageLRU until after we're
-		 * sure the page is not being freed elsewhere -- the
-		 * page release code relies on it.
+		 * Be careful not to clear lru flag until after we're
+		 * sure the folio is not being freed elsewhere -- the
+		 * folio release code relies on it.
  		 */
-		folio = folio_get_nontail_page(page);
-		if (unlikely(!folio))
+		if (unlikely(folio_try_get(folio)))
  			goto isolate_fail;
@@ -1074,39 +1074,41 @@ isolate_migratepages_block(struct compact_control *cc, unsigned long low_pfn,
  			/*
-			 * __PageMovable can return false positive so we need
-			 * to verify it under page_lock.
+			 * __folio_test_movable can return false positive so
+			 * we need to verify it under page_lock.
  			 */
/*




[Index of Archives]     [Linux ARM Kernel]     [Linux ARM]     [Linux Omap]     [Fedora ARM]     [IETF Annouce]     [Bugtraq]     [Linux OMAP]     [Linux MIPS]     [eCos]     [Asterisk Internet PBX]     [Linux API]

  Powered by Linux